#!/usr/bin/env python2.3

# sync_verify.py | Copyright(C), 2004, Enfold Systems, LLC
#
# Enfold Systems, LLC
# 4500 Montrose Blvd., Suite D
# Houston, Texas 77006 USA
# p. +1 713.942.2377 | f. +1 832.201.8856
# www.enfoldsystems.com
# info@enfoldsystems.com
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License as
# published by the Free Software Foundation; either version 2.1 of the
# License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public
# License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this library; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
# For a full copy of the license, please see the included 'LGPL.txt' file.
"""
$Id: repository.py 559 2004-12-03 02:11:23Z jlaplace $
"""

import os, os.path
import sys
from optparse import OptionParser
from BTrees.OOBTree import OOBTree, OOSet, difference, intersection, union
from Acquisition import aq_base
from Products.CMFCore.utils import getToolByName
from Products.ExternalFile.ExternalFile import ExternalFile

def verify(root):
    ct = getToolByName(root, 'portal_catalog')
    purl = getToolByName(root, 'portal_url')
    portal = purl.getPortalObject()
    mt = getToolByName(root, 'portal_managed_files')

    fs = OOSet()
    def fs_collect(options, dirname, filelist):
        for filename in filelist:
            fullpath = os.path.join(dirname, filename)
            if os.path.isfile(fullpath):
                fs.insert(fullpath)

    for repo_name in mt._repositoryList:
        repo = mt.getRepository(repo_name)
        os.path.walk(repo.getPath(), fs_collect, ())

    zodb = OOSet()
    zodb_path = OOBTree()
    def collect(obj, path):
        if isinstance(obj, ExternalFile):
            path = obj.getFilepath()
            zodb.insert(path)
            if not zodb_path.has_key(path):
                zodb_path[path] = []
            zodb_path[path].append(purl.getRelativeContentURL(obj))

    ct.ZopeFindAndApply(obj=portal, search_sub=True,
                        apply_func=collect)

    zodb_orphan = difference(zodb, fs)
    path_orphan = intersection(zodb_path, zodb_orphan)
    zodb_path_orphan = OOSet()
    for fname in path_orphan:
        zodb_path_orphan.update(zodb_path[fname])
    fs_orphan = difference(fs, zodb)
    common = intersection(zodb, fs)
    return common, zodb_orphan, zodb_path_orphan, fs_orphan


if __name__ == '__main__':
    parser = OptionParser()
    parser.add_option('-p', '--path',
                      dest='path',
                      help='Path to a CMF/Plone site inside the ZODB')
    parser.add_option('-v', '--verbose',
                      action='store_const',
                      const=True,
                      dest='verbose',
                      help='Print a detailed summary')
    parser.add_option('-o', '--output',
                      dest='output',
                      help=('Path of a file where the list of filenames '
                            'that are orphaned on the filesystem will '
                            'be stored.'))
    parser.add_option('-f', '--zodb-filepath',
                      dest='zodb_filepath',
                      help=('Path of a file where the list of filenames '
                            'that are orphaned on the ZODB will '
                            'be stored.'))
    parser.add_option('-z', '--zodb-path',
                      dest='zodb_path',
                      help=('Path of a file where the list of paths '
                            'for objects that are orphaned on the '
                            'ZODB will be stored.'))
    (options, args) = parser.parse_args()
    if not options.path:
        parser.print_help()
        sys.exit(1)

    import Zope
    app = Zope.app()
    root = app.unrestrictedTraverse(options.path, default=None)
    if root is None:
        print 'Could not find object at %s' % options.path
    common, zodb_orphan, path_orphan, fs_orphan = verify(root)

    def noop(*args): pass

    if not options.verbose:
        msg = noop
    else:
        def msg(txt):
            if options.verbose:
                print txt

    if options.zodb_path:
        f_zodb_path = open(options.zodb_path, 'wb')
        w_zodb_path = f_zodb_path.write
    else:
        f_zodb_path = None
        w_zodb_path = noop

    if options.zodb_filepath:
        f_zodb_filepath = open(options.zodb_filepath, 'wb')
        w_zodb_filepath = f_zodb_filepath.write
    else:
        f_zodb_filepath = None
        w_zodb_filepath = noop

    if options.output:
        f_output = open(options.output, 'wb')
        w_output = f_output.write
    else:
        f_output = None
        w_output = noop

    if common:
        msg('Files which are present on ZODB and FS')
        for fname in common:
            msg(fname)
        else:
            msg('No common files on ZODB and FS')

    if zodb_orphan:
        msg('Orphaned files on ZODB')
        for fname in zodb_orphan:
            msg(fname)
            w_zodb_filepath(fname)
        msg('Orphaned paths on ZODB')
        for path in path_orphan:
            msg(path)
            w_zodb_path(path)
    else:
        msg('No orphaned files on ZODB')

    if fs_orphan:
        msg('Orphaned files on filesystem')
        for fname in fs_orphan:
            msg(fname)
            w_output(fname)
    else:
        msg('No orphaned files on filesystem')


    if f_output:
        f_output.close()

    if f_zodb_path:
        f_zodb_path.close()

    if f_zodb_filepath:
        f_zodb_filepath.close()
